#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
## This file may be used under the terms of the GNU General Public
## License version 2.0 as published by the Free Software Foundation
## and appearing in the file LICENSE included in the packaging of
## this file. 
##
## This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
## WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
##
## See the NOTICE file distributed with this work for additional
## information regarding copyright ownership.

from string import *
import sys
import re

from util import toolbox
from util import util
from lxml import etree as ET
import time
from datetime import datetime
from dateutil.relativedelta import *
import traceback
from lxml import etree as ElementTree
import os
import threading
import subprocess

class PowerTop(threading.Thread):

    def __init__(self, interval, logger, scriptDir):
        threading.Thread.__init__(self)
        self.interval = interval
        self.logger = logger
        self.scriptDir = scriptDir

        
    def run(self):
        cmd = ["/usr/bin/powertop", "-d", "-t", str(self.interval)]
        self.powertop = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        cmd = ["python", "%s/powertop.py" % self.scriptDir]
        self.powertoplog = subprocess.Popen(cmd, stdin=self.powertop.stdout, stderr=self.logger, stdout=subprocess.PIPE)
        data = self.powertoplog.communicate()[0]
        


def powertoplog(argv):
    dtend = datetime.now()
    epoch = int(time.time())
    dtend_str = datetime.strftime(dtend, "%Y-%m-%d %H:%M:%S")
    
    file = toolbox.openAnything("-")
        
    #http://effbot.org/zone/element.htm
    root = ET.Element("powertop")
    #root.set("xmlns", "http://deskolo.org/2010/08/11/deskolo")
    
    report = ET.SubElement(root, "report")
    
    re_generator = re.compile(r"(PowerTOP)(.*)\n")
    re_interval = re.compile(r"Collecting data for (\d*) seconds")
    re_cstate = re.compile(r"^([^\s]*).*\(([^%]*)%\)\n")
    re_pstate = re.compile(r"^\s+(.*)\s+[^\d]*(.*)%\n")
    re_wakeup = re.compile(r"^\s+([^%]*)%\s\([^\)]*\)\s+([^:]*)\s:\s(.*)\n")

    state = "START"
    
    for line in file:
        
        if state == "START":
            result = re_generator.search(line)
            if result:
                generator = result.group(1) + result.group(2)
                generatorElt = ET.SubElement(report, "generator")
                generatorElt.text = generator
                           
            result = re_interval.search(line)
            if result:
                interval = int(str(result.group(1)))
                
                dtstart = dtend + relativedelta(seconds= -interval)
                dtstart_str = datetime.strftime(dtstart, "%Y-%m-%d %H:%M:%S")     
    
                dtstartElt = ET.SubElement(report, "start")
                dtstartElt.text = dtstart_str 
                
                dtendElt = ET.SubElement(report, "end")
                dtendElt.text = dtend_str
                
                state = "CSTATE"
                
                
            
        if state == "CSTATE":
            result = re_cstate.search(line)
            if result:
                cstate = result.group(1)
                #cstate_desc = re.sub(r"(\s+)", " ", result.group().strip())
                cstate_desc = result.group(2).strip()
                #output += "CSTATE: C" + cstate + ": " + cstate_desc + "\n"
                cstate_elt = ET.SubElement(report, "c-state")
                
                cstate_elt.set("name", cstate)
                cstate_elt.set("percent", cstate_desc)
                #cstate_elt.set("status",status)
                
                try:
                    import deskolo
                    deskolo.logQuantity("/var/lib/deskolo", "powertop", "cstate-%s" % cstate, cstate_desc, 60, epoch)
                except Exception, e:
                    traceback.print_exc(file=sys.stdout)


            if line.find("P-states (frequencies)") == 0:
                state = "PSTATE"
                
            
        elif state == "PSTATE":
            result = re_pstate.search(line)
            if result:
                pstate = result.group(1).strip()
                value = result.group(2).strip()
                #output += "PSTATE: P " + pstate + " " + value
                pstate_elt = ET.SubElement(report, "p-state")
                pstate_elt.set("name", pstate)
                pstate_elt.set("percent", value)
             
             
                pstate = pstate.replace(" ","-")
                try:
                    import deskolo
                    deskolo.logQuantity("/var/lib/deskolo", "powertop", "pstate-%s" % pstate, value, 60, epoch)
                except Exception, e:
                    traceback.print_exc(file=sys.stdout)

             
            
            if line.find("Wakeups-from-idle per second") == 0:
                
                state = "WAKEUPS"
                wakeups = ET.SubElement(report, "wakeups")
                
                re_wakeups_rate = re.compile(r"[^:]*\s:\s([^\s]*).*\n")
                result = re_wakeups_rate.search(line)
                if result:
                    wakeups_rate = result.group(1)
                    wakeups.set("frequency", wakeups_rate)
                    try:
                        import deskolo
                        deskolo.logQuantity("/var/lib/deskolo", "powertop", "wakeups" % cstate, wakupes_rate, 60, epoch)
                    except Exception, e:
                        traceback.print_exc(file=sys.stdout)
                    
        
        elif state == "WAKEUPS":
            result = re_wakeup.search(line)
            if result:
                #output += result.group(1) + result.group(2) + result.group(3) + "\n"
                wakeup = ET.SubElement(wakeups, "wakeup")
                component = result.group(2).strip().replace("<", "").replace(">", "")
                wakeup.set("component", component)
                cause = result.group(3).strip()
                wakeup.set("cause", cause)
                percent = result.group(1).strip()
                wakeup.set("percent", percent)
                
        #print line,
    
    
    # wrap it in an ElementTree instance, and save as XML
    #tree = ET.ElementTree(root)
    #tree.write("powertop.xml")
    
    file.close()
    
    return root

def logXmlRecord(recordstr):
    Etree = ElementTree
    
    path = "%s/xml/powertop/powertop-%s.xml" % ("/var/lib/deskolo", util.getFormattedDate())
    
    if not os.path.exists("%s/xml/powertop" % ("/var/lib/deskolo")):
        os.makedirs("%s/xml/powertop" % ("/var/lib/deskolo"))
    
    if os.path.exists(path) and os.path.getsize(path) > 0:
        doc = Etree.parse(path)
        root = doc.getroot()
    else:
        root = Etree.Element("powertop")
        root.set("xmlns", "http://deskolo.org/2010/08/11/deskolo")
        doc = Etree.ElementTree(root)
    
    newRecord = Etree.fromstring(recordstr)
    
    newReport = newRecord[0]
    root.append(newReport)

    util.indent(root)

    doc.write(path)

if __name__ == "__main__":
    if False:
        print "Usage:"
        print "\tpowertop -d -t 1 | python " + sys.argv[0]
        sys.exit()

    powertop_elt = powertoplog(sys.argv[1:])
    
    powertop_xml = ET.tostring(powertop_elt)
    logXmlRecord(powertop_xml)

